@@ -1,5 +1,6 @@
 import re
 import copy
+from datetime import datetime
 
 class SchemaError(ValueError):
     """errors relating to an invalid schema passed to validate"""
@@ -231,6 +232,37 @@ class SchemaValidator(object):
     validate_minItems = validate_minLength
     validate_maxItems = validate_maxLength
 
+    def validate_format(self, x, fieldname, schema, format_option=None):
+        '''
+        Validates the format of primitive data types
+        '''
+        value = x.get(fieldname)
+
+        date_formats = {
+            'date-time': '%Y-%m-%dT%H:%M:%SZ',
+            'date': '%Y-%m-%d',
+            'time': '%H:%M:%S',
+        }
+
+        if isinstance(value, str) and format_option in date_formats.keys():
+            try:
+                datetime.strptime(value, date_formats[format_option])
+            except ValueError, e:
+                self._error("Value %(value)r of field '%(fieldname)s' is not in '%(format_option)s' format",
+                            value, fieldname, format_option=format_option)
+
+        if format_option == 'utc-millisec':
+            if not isinstance(value, (int,float)):
+                self._error("Value %(value)r of field '%(fieldname)s' is not a number",
+                            value, fieldname)
+
+            if not value > 0:
+                self._error("Value %(value)r of field '%(fieldname)s' is not a positive number",
+                            value, fieldname)
+
+        # TODO: warn about unsupported format ?
+
+
     def validate_pattern(self, x, fieldname, schema, pattern=None):
         '''
         Validates that the given field, if a string, matches the given
@@ -242,6 +274,33 @@ class SchemaValidator(object):
                 self._error("Value %(value)r for field '%(fieldname)s' does not match regular expression '%(pattern)s'",
                             value, fieldname, pattern=pattern)
 
+    def validate_uniqueItems(self, x, fieldname, schema, unique=False):
+        '''
+        Validates that all items in an array instance MUST be unique
+        (contains no two identical values).
+        '''
+
+        if not unique:
+            return # nothing to validate
+
+        values = x.get(fieldname)
+
+        hashables = set()
+        unhashables = []
+
+        for value in values:
+            if isinstance(value, (list, dict)):
+                container, add = unhashables, unhashables.append
+            else:
+                container, add = hashables, hashables.add
+
+            if value in container:
+                self._error(
+                    "Value %(value)r for field '%(fieldname)s' is not unique",
+                    value, fieldname)
+            else:
+                add(value)
+
     def validate_enum(self, x, fieldname, schema, options=None):
         '''
         Validates that the value of the field is equal to one of the
